#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <stdio.h>
#include <dos.h>

template <class vectorvaltype>
class vectortype{
	public:
		vectorvaltype x1, y1, x2, y2, deltax, deltay;

		vectortype(){ x1 = y1 = x2 = y2 = deltax = deltay = 0; }

		void init(vectorvaltype nx1, vectorvaltype ny1, vectorvaltype nx2, vectorvaltype ny2){
			x1 = nx1;
			y1 = ny1;
			x2 = nx2;
			y2 = ny2;
			deltax = x2 - x1;
			deltay = y2 - y1;
		}

		char operator < (float scalar){
			float testval;

			testval = fabs((float)(deltax * deltax) + (float)(deltay * deltay));
			testval = sqrt(testval);

			return testval < scalar;
		}

		char operator > (float scalar){
			float testval;

			testval = fabs((float)(deltax * deltax) + (float)(deltay * deltay));
			testval = sqrt(testval);

			return testval > scalar;
		}

		void operator += (vectortype<vectorvaltype> v2){
			x2 += (vectorvaltype)v2.deltax;
			y2 += (vectorvaltype)v2.deltay;
			deltax += (vectorvaltype)v2.deltax;
			deltay += (vectorvaltype)v2.deltay;
		}

		void operator -= (vectortype<vectorvaltype> v2){
			x2 -= (vectorvaltype)v2.deltax;
			y2 -= (vectorvaltype)v2.deltay;
			deltax -= (vectorvaltype)v2.deltax;
			deltay -= (vectorvaltype)v2.deltay;
		}

		vectortype operator - (vectortype<vectorvaltype> v2){
			vectortype<vectorvaltype> returnval;
			returnval.x1 = (vectorvaltype)x1;
			returnval.y1 = (vectorvaltype)y1;
			returnval.deltax = deltax - (vectorvaltype)v2.deltax;
			returnval.deltay = deltay - (vectorvaltype)v2.deltay;
			returnval.x2 = x2 - (vectorvaltype)v2.deltax;
			returnval.y2 = y2 - (vectorvaltype)v2.deltay;

			return returnval;
		}

		vectortype operator + (vectortype<vectorvaltype> v2){
			vectortype<vectorvaltype> returnval;
			returnval.x1 = (vectorvaltype)x1;
			returnval.y1 = (vectorvaltype)y1;
			returnval.deltax = deltax + (vectorvaltype)v2.deltax;
			returnval.deltay = deltay + (vectorvaltype)v2.deltay;
			returnval.x2 = x2 + (vectorvaltype)v2.deltax;
			returnval.y2 = y2 + (vectorvaltype)v2.deltay;

			return returnval;
		}

		void operator *= (float scalar){
			deltax = (vectorvaltype)((float)deltax * scalar);
			deltay = (vectorvaltype)((float)deltay * scalar);
			x2 = x1 + deltax;
			y2 = y1 + deltay;
		}

		vectortype operator * (float scalar){
			vectortype<vectorvaltype> returnval;
			returnval.x1 = (vectorvaltype)x1;
			returnval.y1 = (vectorvaltype)y1;
			returnval.deltax = (vectorvaltype)((float)deltax * scalar);
			returnval.deltay = (vectorvaltype)((float)deltay * scalar);
			returnval.x2 = returnval.x1 + returnval.deltax;
			returnval.y2 = returnval.y1 + returnval.deltay;

			return returnval;
		}

};

template <class pointvaltype>
class pointtype{
	public:
		pointvaltype x, y;

		pointtype(){ x = y = 0;	}
		void setpos(pointvaltype newx, pointvaltype newy){ x = newx; y = newy; }

		void operator += (vectortype<pointvaltype> vector){
			x += (pointvaltype)vector.deltax;
			y += (pointvaltype)vector.deltay;
		}

		void operator -= (vectortype<pointvaltype> vector){
			x -= (pointvaltype)vector.deltax;
			y -= (pointvaltype)vector.deltay;
		}

		pointtype<pointvaltype> operator + (vectortype<pointvaltype> vector){
			pointtype<pointvaltype> returnval;
			returnval.x = x + (pointvaltype)vector.deltax;
			returnval.y = y + (pointvaltype)vector.deltay;
			return returnval;
		}

		pointtype<pointvaltype> operator - (vectortype<pointvaltype> vector){
			pointtype<pointvaltype> returnval;
			returnval.x = x - (pointvaltype)vector.deltax;
			returnval.y = y - (pointvaltype)vector.deltay;
			return returnval;
		}
};

typedef struct rangetype{ /** this structure is only used for polygons **/
	int x1, x2;
	char doneonce;
};

/**************************** declarations *****************************/
unsigned char readkbd(void);
void erase(int x1, int y1, int x2, int y2, int xoffset, int yoffset, unsigned char far *pic);
char touching(int x1, int y1, char *pic1, int x2, int y2, char *pic2, char mode);
void cls(void);
char load_font(unsigned char *name);
void unload_font(void);
void save_font(unsigned char *name);
void drawtext(int x, int y, unsigned char *string, unsigned char colour, char mode);
void loadpic(char far *image, int arraysize, unsigned char *filename);
void savepic(char far *image, int arraysize, unsigned char *filename);
void circle(int cx, int cy, int radius, unsigned char colour, char mode);
template <class pointvaltype>
void put(pointtype<pointvaltype> point, unsigned char far *picture, int mode);
void put(int x1, int y1, unsigned char far *picture, int mode);
void get(int x1, int y1, int x2, int y2, unsigned char far *picture);
void box(int x1, int y1, int x2, int y2, int colour, char mode);
void line(int x1, int y1, int x2, int y2, unsigned char colour, char mode);
char getpixel(int x, int y);
char sgn(int val);
void putpixel(int x, int y, unsigned char colour, char mode);
void locate(char x, char y);
int mousex(void);
int mousey(void);
void showmouse(int onoff);
void initmouse(void);
char buttonpressed(void);
void initscrn(char mode);
void readmousediff(void);
char blackcharstatus(void);
void setmousepos(int x, int y);
void setmouselimits(int x1, int y1, int x2, int y2);
char turnoffmouse(void);
void palette(int c, char r, char g, char b);
void set_palette(char *palette);
char kbscan(void);
unsigned int counter(void);
char menuscreen(char *title, char *background, int numoptions, ...);
char choice(char *title, int numoptions, ...);
unsigned char *taketext(unsigned char *title, char *buttonused, char numbuttons, ...);
unsigned char choose(unsigned char *title, char numbuttons, ...);
void writebit(FILE *fout, char bitval);
char readbit(FILE *fin);
void rotput(int cx, int cy, int xoffset, int yoffset, float ang, float scale, unsigned char far *pic, char mode);
void roterase(int x1, int y1, int x2, int y2, int cx, int cy, float ang, float scale, unsigned char far * pic);
void scaleput(int x1, int y1, float xscale, float yscale, unsigned char far *pic, char mode);
void calcrotarea(int centerx, int centery, int xoffset, int yoffset, float ang, float scale, unsigned char far *pic, int *x1, int *y1, int *x2, int *y2);
void calccorners(int centerx, int centery, int xoffset, int yoffset, float ang, float scale, unsigned char far *pic, int *px, int *py);
void putfont(int x1, int y1, unsigned char picnum, unsigned char colour, char mode);
unsigned char csmode(void);
void set_keyboard_rate(unsigned char rate, unsigned char delay);
void hline(int x1, int x2, int y, unsigned char colour, char mode);
void set_count_rate(int rate);
void drawchar(unsigned char c);
void compare_range(int x1, int y1, int x2, int y2, int miny, rangetype *range);
char polygon(int *x, int *y, int numpoints, unsigned char colour, char mode);

/**************************** definitions ******************************/
#define tabsize 4
#define pi 3.141592653589793238462643383279502884197169399375

/************************** Global Variables ***************************/
/************* An interesting fact: CGA starts at B8000000 *************/
unsigned char far *vmem = (unsigned char far *)0xA0000000;
vectortype<int> mousediff;
unsigned char **font = NULL, xfontsize, yfontsize;

typedef struct particletype{
	vectortype<float> velocity;
	pointtype<float> position;
	float mass;
};

#define numparticles 50

void main(void){
	vectortype<float> tempvector;
	float deltax, deltay, dx, dy, hypotenuse;
	int n, m;
	particletype particle[numparticles];
	unsigned char smode;

	smode = csmode();
	initscrn(0x13);
	if(csmode() == 0x13){	/** screen successfully initialized **/
		initmouse();
		for(n = 32; n < 64; n++){
			palette(n, n, 0, 64 - n);
		}

		particle[0].position.x = mousex();
		particle[0].position.y = mousey();
		particle[0].mass = 50;
		for(n = 1; n < numparticles; n++){
			particle[n].position.x = rand() % 320;
			particle[n].position.y = rand() % 200;
			particle[n].mass = (float)(rand() % 10 + 6) / 16;
		}

		while(!kbhit()){
			putpixel(particle[0].position.x, particle[0].position.y, 0, 0);
			particle[0].position.x = mousex();
			particle[0].position.y = mousey();
			putpixel(particle[0].position.x, particle[0].position.y, 14, 0);

			for(n = 1; n < numparticles; n++){
				putpixel(particle[n].position.x, particle[n].position.y, 0, 0);
				//putpixel(particle[n].position.x, particle[n].position.y, 0, 0);
				deltax = 0;
				deltay = 0;
				for(m = 0; m < numparticles; m++){
					dx = particle[m].position.x - particle[n].position.x;
					dy = particle[m].position.y - particle[n].position.y;
					dx /= 10;
					dy /= 10;
					hypotenuse = sqrt(dx * dx + dy * dy);
					if(hypotenuse < 1.) hypotenuse = 1.;
					hypotenuse *= particle[n].mass / particle[m].mass;
					deltax += 10. * dx / hypotenuse;
					deltay += 10. * dy / hypotenuse;
				}
				deltax /= numparticles;
				deltay /= numparticles;
				deltax /= 20;
				deltay /= 20;
				tempvector.init(0, 0, deltax, deltay);
				if(particle[n].velocity > 1.) particle[n].velocity *= .98;
				//particle[n].velocity += tempvector;
				particle[n].velocity += tempvector;// * .5;

				particle[n].position += particle[n].velocity;
				putpixel(particle[n].position.x, particle[n].position.y, 32 + 32 * particle[n].mass, 0);

			}
		}

		while(kbhit()) getch();

		initscrn(smode);
	}else{
		printf("Failurre initializing screen.\n");
	}
}

/**************************** Keyboard I/O *****************************/
void set_keyboard_rate(unsigned char rate, unsigned char delay){
	delay %= 4;
	rate %= 32;
	asm{
		mov ah, 0x3
		mov al, 0x5
		mov bh, delay
		mov bl, rate
		int 0x16
	}
}

unsigned char readkbd(void){
	if(!kbhit()) asm int 9; /* kills keyboard delay */
	if(kbhit())
		return getch();
	else
		return NULL;
}

char blackcharstatus(void){
	char returnval;
	asm{
		mov ah, 0x2
		int 0x16
		mov returnval, al
	}
	return returnval;
}

char kbscan(void){
	return inp(0x60);
}

void drawchar(unsigned char c){
	asm{
		mov ah, 0x2
		mov dl, c
		int 0x21
	}
}

/**************************** Mouse I/O *****************************/
void readmousediff(void){
	int x, y;
	asm{
		mov ax, 0xB
		int 0x33
		mov x, cx
		mov y, dx
	}
	mousediff.x2 = mousediff.deltax = x;
	mousediff.y2 = mousediff.deltay = y;
	mousediff.x1 = mousediff.y1 = 0;
}

char buttonpressed(void){
	int returnval;
	asm{
		mov ax, 0x3
		int 0x33
		mov returnval, bx
	}
	return returnval;
}

int mousex(void){
	int returnval;
	asm{
		mov ax, 0x3
		int 0x33
		mov returnval, cx
	}
	return returnval >> 1;
}

int mousey(void){
	int returnval;
	asm{
		mov ax, 0x3
		int 0x33
		mov returnval, dx
	}
	return returnval;

}

void initmouse(void){
	asm{
		mov ax, 0
		int 0x33
	}
}

void showmouse(int onoff){
	onoff = (onoff == 0) + 1;
	asm{
		mov ax, onoff
		int 0x33
	}
}

char turnoffmouse(void){
	int returnval;
	asm{
		mov ax, 0x1F
		int 0x33
		mov returnval, ax
	}
	return returnval;
}

void setmousepos(int x, int y){
	asm{
		mov ax, 0x4
		mov cx, x
		shl cx, 1
		mov dx, y
		int 0x33
	}
}

void setmouselimits(int x1, int y1, int x2, int y2){
	asm{
		/*********** set x range **********/
		mov ax, 0x7
		mov cx, x1
		mov dx, x2
		shl cx, 1
		shl dx, 1
		int 0x33

		/*********** set y range **********/
		mov ax, 0x8
		mov cx, y1
		mov dx, y2
		int 0x33
	}
}

/****************************** Misc ******************************/
void locate(char x, char y){
	asm{
		mov ah, 2
		mov dh, y
		mov dl, x
		mov bh, 0
		int 0x10
	}
}

char sgn(int val){
	if(val > 0) return 1;
	return -(val < 0);
}

unsigned int counter(void){
	return peek(0x40, 0x6C);
}

void set_count_rate(int rate){
	_asm cli
	if(rate < 19) rate = 19;
	rate = (rate == 18) ? 0 : ((unsigned int) (1193180L / ((unsigned long)rate)));
	asm{
		mov dx, 0x43
		mov al, 0x34
		out dx, al
		mov dx, 0x40
		mov ax, rate
		out dx, al
		mov al, ah
		out dx, al
		sti
	}
}

/****************************** Graphics ******************************/
void initscrn(char mode){
	asm{
		mov ah, 0
		mov al, mode
		int 0x10
	}
}

unsigned char csmode(void){
	unsigned char returnval;
	asm{
		mov ah, 0xF
		int 0x10
		mov returnval, al
	}
	return returnval;
}

void putpixel(int x, int y, unsigned char colour, char mode){
	if((x >= 0) && (x <= 319) && (y >= 0) && (y <= 199))
		switch(mode){
			case 0: vmem[(y << 8) + (y << 6) + x]  = colour; break;
			case 1: vmem[(y << 8) + (y << 6) + x] ^= colour; break;
			case 2: vmem[(y << 8) + (y << 6) + x] |= colour; break;
			case 3: vmem[(y << 8) + (y << 6) + x] &= colour; break;
			case 4:
				if(colour) vmem[(y << 8) + (y << 6) + x] = colour;
				break;
			case 5:
				if(!vmem[(y << 8) + (y << 6) + x])
					vmem[(y << 8) + (y << 6) + x] = colour;
				break;
			case 6: vmem[(y << 8) + (y << 6) + x] += colour; break;
		}
}

char getpixel(int x, int y){
	if((x >= 0) && (x <= 319) && (y >= 0) && (y <= 199))
		return vmem[(y << 8) + (y << 6) + x];
	else
		return 0;
}

void line(int x1, int y1, int x2, int y2, unsigned char colour, char mode){
	int n, deltax, deltay, sgndeltax, sgndeltay, deltaxabs, deltayabs, x, y, drawx, drawy;

	deltax = x2 - x1;
	deltay = y2 - y1;
	deltaxabs = abs(deltax);
	deltayabs = abs(deltay);
	sgndeltax = sgn(deltax);
	sgndeltay = sgn(deltay);
	x = deltayabs >> 1;
	y = deltaxabs >> 1;
	drawx = x1;
	drawy = y1;

	putpixel(drawx, drawy, colour, mode);

	if(deltaxabs >= deltayabs){
		for(n = 0; n < deltaxabs; n++){
			y += deltayabs;
			if(y >= deltaxabs){
				y -= deltaxabs;
				drawy += sgndeltay;
			}
			drawx += sgndeltax;
			putpixel(drawx, drawy, colour, mode);
		}
	}else{
		for(n = 0; n < deltayabs; n++){
			x += deltaxabs;
			if(x >= deltayabs){
				x -= deltayabs;
				drawx += sgndeltax;
			}
			drawy += sgndeltay;
			putpixel(drawx, drawy, colour, mode);
		}
	}
}

void box(int x1, int y1, int x2, int y2, int colour, char mode){
	int x, y, n;
	if(x1 < 0) x1 = 0; if(x1 > 319) x1 = 319;
	if(y1 < 0) y1 = 0; if(y1 > 199) y1 = 199;
	if(x2 < 0) x2 = 0; if(x2 > 319) x2 = 319;
	if(y2 < 0) y2 = 0; if(y2 > 199) y2 = 199;
	if(x1 > x2) { x1 ^= x2; x2 ^= x1; x1 ^= x2; }
	if(y1 > y2) { y1 ^= y2; y2 ^= y1; y1 ^= y2; }

	if(mode == 4){
		for(x = x1; x <= x2; x++){
			vmem[(y1 << 8) + (y1 << 6) + x] = colour;
			vmem[(y2 << 8) + (y2 << 6) + x] = colour;
		}
		for(y = y1; y <= y2; y++){
			vmem[(y << 8) + (y << 6) + x1] = colour;
			vmem[(y << 8) + (y << 6) + x2] = colour;
		}

	}else{
		for(x = x1; x <= x2; x++){
			for(y = y1; y <= y2; y++){
				switch(mode){
					case 0: vmem[(y << 8) + (y << 6) + x]  = colour; break;
					case 1:	vmem[(y << 8) + (y << 6) + x] ^= colour; break;
					case 2: vmem[(y << 8) + (y << 6) + x] |= colour; break;
					case 3: vmem[(y << 8) + (y << 6) + x] &= colour; break;
					case 5:
						n = (y << 8) + (y << 6) + x;
						vmem[n] += (!vmem[n]) * colour;
					break;
					case 6: vmem[(y << 8) + (y << 6) + x] += colour; break;
				}
			}
		}
	}
}

void circle(int cx, int cy, int radius, unsigned char colour, char mode){
	int ydist, ydist2, xdist, x, y, dx2;
	int x1;
	if(radius < 1) radius *= -1;
	if(!radius) radius++;
	x1 = cx - radius;
	for(x = x1; x <= cx; x += 1){
		xdist = abs(x - cx);
		ydist = sqrt(radius * radius - xdist * xdist);
		ydist2 = sqrt(radius * radius - (xdist - 1) * (xdist - 1));
		ydist2 += !xdist;
		if(mode == 4){
			if(abs(ydist2 - ydist) > 1){
				for(y = ydist; y < ydist2; y++){
					if((x >= 0) && (x <= 319)){
						if((cy + y >= 0) && (cy + y <= 199)) vmem[x + 320 * (cy + y)] = colour;
						if((cy - y >= 0) && (cy - y <= 199)) vmem[x + 320 * (cy - y)] = colour;
					}
					dx2 = cx + xdist;
					if((dx2 >= 0) && (dx2 <= 319)){
						if((cy + y >= 0) && (cy + y <= 199)) vmem[dx2 + 320 * (cy + y)] = colour;
						if((cy - y >= 0) && (cy - y <= 199)) vmem[dx2 + 320 * (cy - y)] = colour;
					}
				}
			}else{
				dx2 = cx + xdist;
				y = cy + ydist;
				if((y >= 0) && (y <= 199)){
					if((x >= 0) && (x <= 319)) vmem[x + 320 * y] = colour;
					if((dx2 >= 0) && (dx2 <= 319)) vmem[dx2 + 320 * y] = colour;
				}
				y = cy - ydist;
				if((y >= 0) && (y <= 199)){
					if((x >= 0) && (x <= 319)) vmem[x + 320 * y] = colour;
					if((dx2 >= 0) && (dx2 <= 319)) vmem[dx2 + 320 * y] = colour;
				}
			}
		}else{
			box(x, cy + ydist2, x, cy - ydist2, colour, mode);
			dx2 = cx + xdist;
			if(xdist > 0)box(dx2, cy + ydist2, dx2, cy - ydist2, colour, mode);
		}
	}
}

void compare_range(int x1, int y1, int x2, int y2, int miny, rangetype *range){
	int n, deltax, deltay, sgndeltax, sgndeltay, index;
	int deltaxabs, deltayabs, x, y, drawx, drawy;

	deltax = x2 - x1;
	deltay = y2 - y1;
	deltaxabs = abs(deltax);
	deltayabs = abs(deltay);
	sgndeltax = sgn(deltax);
	sgndeltay = sgn(deltay);
	x = deltayabs >> 1;
	y = deltaxabs >> 1;
	drawx = x1;
	drawy = y1;

	/***** compare current position with range ****/
	index = drawy - miny;
	if(range[index].doneonce){
		if(range[index].x1 > drawx)
			range[index].x1 = drawx;
		if(range[index].x2 < drawx)
			range[index].x2 = drawx;
	}else{
		range[index].x1 = drawx;
		range[index].x2 = drawx;
		range[index].doneonce = 1;
	}

	if(deltaxabs >= deltayabs){
		for(n = 0; n < deltaxabs; n++){
			y += deltayabs;
			if(y >= deltaxabs){
				y -= deltaxabs;
				drawy += sgndeltay;
			}
			drawx += sgndeltax;

			/***** compare current position with range ****/
			index = drawy - miny;
			if(range[index].doneonce){
				if(range[index].x1 > drawx)
					range[index].x1 = drawx;
				if(range[index].x2 < drawx)
					range[index].x2 = drawx;
			}else{
				range[index].x1 = drawx;
				range[index].x2 = drawx;
				range[index].doneonce = 1;
			}
		}
	}else{
		for(n = 0; n < deltayabs; n++){
			x += deltaxabs;
			if(x >= deltayabs){
				x -= deltayabs;
				drawx += sgndeltax;
			}
			drawy += sgndeltay;

			/***** compare current position with range ****/
			index = drawy - miny;
			if(range[index].doneonce){
				if(range[index].x1 > drawx)
					range[index].x1 = drawx;
				if(range[index].x2 < drawx)
					range[index].x2 = drawx;
			}else{
				range[index].x1 = drawx;
				range[index].x2 = drawx;
				range[index].doneonce = 1;
			}
		}
	}
}

char polygon(int *x, int *y, int numpoints, unsigned char colour, char mode){
	/*******
		This function will work perfectly with any natural number of points
		under one condition:  No two points on the polygon can define a line
		segment that goes outside the polygon.  Otherwise, it will probably
		draw outside the polygon as well, depending on the relative positions
		of the line segments.  An out line of the letter 'M', for instance,
		would be drawn as a solid rectangle.
	*******/
	rangetype *range;
	int n, m, miny, maxy, ysize;
	char returnval = 0;

	if(mode == 4){
		for(n = 0; n < numpoints; n++)
			line(x[n], y[n], x[(n + 1) % numpoints], y[(n + 1) % numpoints],
				 colour, 0);
	}else{

		miny = maxy = y[0];
		for(n = 1; n < numpoints; n++){
			if(y[n] < miny) miny = y[n];
			if(y[n] > maxy) maxy = y[n];
		}
		ysize = maxy - miny + 1;

		range = (rangetype *)malloc(ysize * sizeof(rangetype));
		if(range){
			/** initialize ranges **/
			for(n = 0; n < ysize; n++) range[n].doneonce = 0;

			/** calculate ranges **/
			for(n = 0; n < numpoints; n++){
				compare_range(x[n], y[n], x[(n + 1) % numpoints],
							  y[(n + 1) % numpoints], miny, range);
			}

			/** draw polygon **/
			if(miny <= 199 && maxy >= 0){
				m = 0;
				for(n = miny; n <= maxy; n++){
					if(n >= 0 && n < 200)
						hline(range[m].x1, range[m].x2,
							  n, colour, mode);
					m++;
				}
			}
			free(range);
		}else returnval = 1;
	}
	return returnval;
}

void hline(int x1, int x2, int y, unsigned char colour, char mode){
	/** WARNING: when used in drawing mode 0, this does not check for screen
		limits, it assumes that the calling function does.  In any other
		modes, putpixel checks it instead.
	**/
	int n;
	if(x1 > x2){ x1 ^= x2; x2 ^= x1; x1 ^= x2; }
	if(!mode)
		_fmemset(&vmem[x1 + 320 * y], colour, x2 - x1 + 1);
	else
		for(n = x1; n <= x2; n++)
			putpixel(n, y, colour, mode);
}

void set_palette(char *palette){
	int n;

	outp(0x03C8, 0); /* Tell the VGA that palette data is coming. */

	for(n = 0; n < 768; n++)
		outp(0x03C9, palette[n]);    /* write the data */
}


void palette(int c, char r, char g, char b){
	r %= 64;
	g %= 64;
	b %= 64;
	asm{
		mov ah, 0x10
		mov al, 0x10
		mov bx, c
		mov ch, g
		mov cl, b
		mov dh, r
		int 0x10
	}
}

void cls(void){
	_fmemset(vmem, 0, 64000);
}

void drawtext(int x, int y, unsigned char *string, unsigned char colour, char mode){
	int length = strlen(string), n, m, initx = x;
	char charval, junk[100], c, repeat;

	for(n = 0; n < length; n++){
		do{
			charval = string[n];
			repeat = 1;
			switch(charval){
				case '\r':
					x = initx; /** please note: break is gone for a reason. */
				case '\n':
					y += yfontsize;
					break;
				case '\t':
					x += tabsize * xfontsize;
					break;
				case '\b':
					x -= xfontsize;
					break;
				case '\v':
					y += 4 * yfontsize;
					break;
				default:
					repeat = 0;
			}
			n += repeat;
		}while(repeat);
		putfont(x, y, charval, colour, mode);
		x += xfontsize;
	}
}

void get(int x1, int y1, int x2, int y2, unsigned char far *picture){
	int n = 2, x, y;
	if(x1 > x2){ x1 ^= x2; x2 ^= x1; x1 ^= x2; }
	if(y1 > y2){ y1 ^= y2; y2 ^= y1; y1 ^= y2; }

	picture[0] = x2 - x1 + 1;
	picture[1] = y2 - y1 + 1;
	for(y = y1; y <= y2; y++){
		for(x = x1; x <= x2; x++){
			if((x >= 0) || (x <= 319) || (y >= 0) || (y <= 199))
				picture[n] = vmem[(y << 8) + (y << 6) + x];
			else
				picture[n] = 0;

			n++;
		}
	}
}

void putfont(int x1, int y1, unsigned char picnum, unsigned char colour, char mode){
	int n = 2, x, y, x2, y2;
	unsigned char *picture;
	unsigned int dy, madd;

	if(font){
		picture = font[picnum];
		x2 = x1 + picture[0] - 1;
		y2 = y1 + picture[1] - 1;
		mode %= 7;

		switch(mode){
			case 0:
				for(y = y1; y <= y2; y++){
					if(y >= 0 && y <= 199){
						dy = (y << 8) + (y << 6);
						for(x = x1; x <= x2; x++){
							if((x >= 0) && (x <= 319)){
								madd = dy + x;
								vmem[madd]  = (picture[n] != 0) * colour;
							}
							n++;
						}
					}else{
						n += picture[0];
					}
				}
				break;

			case 1:
				for(y = y1; y <= y2; y++){
					if(y >= 0 && y <= 199){
						dy = (y << 8) + (y << 6);
						for(x = x1; x <= x2; x++){
							if((x >= 0) && (x <= 319)){
								madd = dy + x;
								vmem[madd] ^= (picture[n] != 0) * colour;
							}
							n++;
						}
					}else{
						n += picture[0];
					}
				}
				break;

			case 2:
				for(y = y1; y <= y2; y++){
					if(y >= 0 && y <= 199){
						dy = (y << 8) + (y << 6);
						for(x = x1; x <= x2; x++){
							if((x >= 0) && (x <= 319)){
								madd = dy + x;
								vmem[madd] |= (picture[n] != 0) * colour;
							}
							n++;
						}
					}else{
						n += picture[0];
					}
				}
				break;

			case 3:
				for(y = y1; y <= y2; y++){
					if(y >= 0 && y <= 199){
						dy = (y << 8) + (y << 6);
						for(x = x1; x <= x2; x++){
							if((x >= 0) && (x <= 319)){
								madd = dy + x;
								vmem[madd] &= (picture[n] != 0) * colour;
							}
							n++;
						}
					}else{
						n += picture[0];
					}
				}
				break;

			case 4:
				for(y = y1; y <= y2; y++){
					if(y >= 0 && y <= 199){
						dy = (y << 8) + (y << 6);
						for(x = x1; x <= x2; x++){
							if((x >= 0) && (x <= 319)){
								if(picture[n]){
									madd = dy + x;
									vmem[madd] = colour;
								}
							}
							n++;
						}
					}else{
						n += picture[0];
					}
				}
				break;

			case 5:
				for(y = y1; y <= y2; y++){
					if(y >= 0 && y <= 199){
						dy = (y << 8) + (y << 6);
						for(x = x1; x <= x2; x++){
							if((x >= 0) && (x <= 319)){
								madd = dy + x;
								if(!vmem[madd])
									vmem[madd]  = (picture[n] != 0) * colour;
							}
							n++;
						}
					}else{
						n += picture[0];
					}
				}
				break;

			case 6:
				for(y = y1; y <= y2; y++){
					if(y >= 0 && y <= 199){
						dy = (y << 8) + (y << 6);
						for(x = x1; x <= x2; x++){
							if((x >= 0) && (x <= 319)){
								madd = dy + x;
								vmem[madd] += (picture[n] != 0) * colour;
							}
							n++;
						}
					}else{
						n += picture[0];
					}
				}
				break;
		}
	}
}

template <class pointvaltype>
void put(pointtype<pointvaltype> point, unsigned char far *picture, int mode){
	unsigned int ymempos;
	int x1 = point.x, y1 = point.y;
	int n = 2, x, y;
	int x2 = x1 + picture[0] - 1;
	int y2 = y1 + picture[1] - 1;

	switch(mode){
		case 0:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x]  = picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 1:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x] ^= picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 2:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x] |= picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 3:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x] &= picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 4:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319 && picture[n])
							vmem[ymempos + x] = picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 5:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319 && !vmem[ymempos + x])
							vmem[ymempos + x] = picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 6:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x] += picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;
	}
}

void put(int x1, int y1, unsigned char far *picture, int mode){
	unsigned int ymempos;
	int n = 2, x, y;
	int x2 = x1 + picture[0] - 1;
	int y2 = y1 + picture[1] - 1;

	switch(mode){
		case 0:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x]  = picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 1:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x] ^= picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 2:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x] |= picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 3:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x] &= picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 4:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319 && picture[n])
							vmem[ymempos + x] = picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 5:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319 && !vmem[ymempos + x])
							vmem[ymempos + x] = picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;

		case 6:
			for(y = y1; y <= y2; y++){
				if(y >= 0 && y <= 199){
					ymempos = (y << 8) + (y << 6);
					for(x = x1; x <= x2; x++){
						if(x >= 0 && x <= 319)
							vmem[ymempos + x] += picture[n];
						n++;
					}
				}else{
					n += picture[0];
				}
			}
			break;
	}
}

void scaleput(int x1, int y1, float xscale, float yscale, unsigned char far *pic, char mode){
	int x, y, px, py, my, cy;
	const int xsize = (float)pic[0] / xscale;
	const int ysize = (float)pic[1] / yscale;
	const int ixscale = xscale * 256;
	const int iyscale = yscale * 256;

	for(y = 0; y <= ysize; y++){
		py = y * iyscale >> 8;
		my = py * pic[0];
		cy = y + y1;
		for(x = 0; x <= xsize; x++){
			px = x * ixscale >> 8;
			putpixel(x + x1, cy, pic[px + my + 2], mode);
		}
	}
}

void calcrotarea(int centerx, int centery, int xoffset, int yoffset, float ang, float scale, unsigned char far *pic, int *x1, int *y1, int *x2, int *y2){
	const int xsize = pic[0], ysize = pic[1];
	int px[4], py[4], minx, miny, maxx, maxy;
	int n, tmp, rotx;
	const int SINE = sin(ang) * 256;
	const int COSINE = cos(ang) * 256;

	scale += !scale;   /**** prevents division by zero ******/

	px[2] = px[0] = -xoffset;
	py[1] = py[0] = -yoffset;
	px[1] = px[3] = px[0] + xsize - 1;
	py[2] = py[3] = py[0] + ysize - 1;

	for(n = 0; n < 4; n++){
		tmp = (px[n] * COSINE - py[n] * SINE) >> 8;
		py[n] = (px[n] * SINE + py[n] * COSINE) >> 8;
		px[n] = tmp;
		px[n] += sgn(px[n]);
		py[n] += sgn(py[n]);
		px[n] *= scale;
		py[n] *= scale;
		px[n] += centerx;
		py[n] += centery;

		if(px[n] < minx || !n) minx = px[n];
		if(py[n] < miny || !n) miny = py[n];
		if(px[n] > maxx || !n) maxx = px[n];
		if(py[n] > maxy || !n) maxy = py[n];
	}

	if(minx < 0) minx = 0;		if(maxx < 0) maxx = 0;
	if(miny < 0) miny = 0;		if(maxy < 0) maxy = 0;
	if(minx > 319) minx = 319;	if(maxx > 319) maxx = 319;
	if(miny > 199) miny = 199;	if(maxy > 199) maxy = 199;
	*x1 = minx;
	*y1 = miny;
	*x2 = maxx;
	*y2 = maxy;
}

void calccorners(int centerx, int centery, int xoffset, int yoffset, float ang, float scale, unsigned char far *pic, int *px, int *py){
	const int xsize = pic[0], ysize = pic[1];
	int n, tmp;
	const int SINE = sin(ang) * 256 * (scale + !scale);
	const int COSINE = cos(ang) * 256 * (scale + !scale);

	scale += !scale;   /**** prevents division by zero ******/

	px[2] = px[0] = -xoffset;
	py[1] = py[0] = -yoffset;
	px[1] = px[3] = px[0] + xsize;
	py[2] = py[3] = py[0] + ysize;

	for(n = 0; n < 4; n++){
		tmp = (px[n] * COSINE - py[n] * SINE) >> 8;
		py[n] = (px[n] * SINE + py[n] * COSINE) >> 8;
		px[n] = tmp;
		px[n] += centerx;
		py[n] += centery;
	}
}

void rotput(int centerx, int centery, int xoffset, int yoffset, float ang, float scale, unsigned char far *pic, char mode){
/***
	As !!UUUUUUGGGGGLLLLLLYYYYYYY!! as this is with the case structure around
	the for loops - rather than vice versa - it's much faster, which is an
	absolute necessity.

	The chunks of code defined here at the top of the function are defined
	so as to make the case structure more ledgible, as these are both
	used in each case.

	Please note: The braces around each asm statement in the second definition
	are there because without them, asm requires a newline character, which
	I can't add as part of the #define.
***/


#define HANDLE_MATH_IN_X_LOOP 					\
	deltax = x - centerx;						\
	rotx = deltax * COSINE2 - deltaymulSINE2;	\
	roty = deltax * SINE2 + deltaymulCOSINE2;	\
	rotx >>= 8;									\
	roty >>= 8;									\
	rotx += xoffset;							\
	roty += yoffset;

#define HANDLE_MATH_IN_Y_LOOP 			\
	asm{mov ax, y}						\
	asm{mov bx, centery}				\
	asm{sub ax, bx}						\
	asm{mov deltay, ax}					\
	deltaymulSINE2 = deltay * SINE2;	\
	deltaymulCOSINE2 = deltay * COSINE2;\
	ymul320 = (y << 8) + (y << 6);

	const int xsize = pic[0], ysize = pic[1];
	int px[4], py[4], minx, miny, maxx, maxy;
	int deltax, deltay, n, tmp, rotx, roty, x, y;
	const int SINE = sin(ang) * 256 * (scale + !scale);
	const int COSINE = cos(ang) * 256 * (scale + !scale);
	const int SINE2 = sin(-ang) * 256 / (scale + !scale);
	const int COSINE2 = cos(-ang) * 256 / (scale + !scale);
	int deltaymulSINE2, deltaymulCOSINE2, ymul320;

	scale += !scale;   /**** prevents division by zero ******/

	px[2] = px[0] = -xoffset;
	py[1] = py[0] = -yoffset;
	px[1] = px[3] = px[0] + xsize;
	py[2] = py[3] = py[0] + ysize;

	for(n = 0; n < 4; n++){
		tmp = (px[n] * COSINE - py[n] * SINE) >> 8;
		py[n] = (px[n] * SINE + py[n] * COSINE) >> 8;
		px[n] = tmp;
		px[n] += centerx;
		py[n] += centery;

		if(px[n] < minx || !n) minx = px[n];
		if(py[n] < miny || !n) miny = py[n];
		if(px[n] > maxx || !n) maxx = px[n];
		if(py[n] > maxy || !n) maxy = py[n];
	}

	if(minx < 0) minx = 0;		if(maxx < 0) maxx = 0;
	if(miny < 0) miny = 0;		if(maxy < 0) maxy = 0;
	if(minx > 319) minx = 319;	if(maxx > 319) maxx = 319;
	if(miny > 199) miny = 199;	if(maxy > 199) maxy = 199;

	maxx++; /* The ++'s here let me use a < comaprison in the for loops, */
	maxy++; /* rather than a <=, which is a tad slower. */

	switch(mode){
		case 0:
			for(y = miny; y < maxy; y++){
				HANDLE_MATH_IN_Y_LOOP
				for(x = minx; x < maxx; x++){
					HANDLE_MATH_IN_X_LOOP
					if(rotx >= 0 && rotx < xsize && roty >= 0 && roty < ysize){
						vmem[x + ymul320] = pic[2 + rotx + xsize * roty];
					}
				}
			}
			break;

		case 1:
			for(y = miny; y < maxy; y++){
				HANDLE_MATH_IN_Y_LOOP
				for(x = minx; x < maxx; x++){
					HANDLE_MATH_IN_X_LOOP
					if(rotx >= 0 && rotx < xsize && roty >= 0 && roty < ysize){
						vmem[x + ymul320] ^= pic[2 + rotx + xsize * roty];
					}
				}
			}
			break;

		case 2:
			for(y = miny; y < maxy; y++){
				HANDLE_MATH_IN_Y_LOOP
				for(x = minx; x < maxx; x++){
					HANDLE_MATH_IN_X_LOOP
					if(rotx >= 0 && rotx < xsize && roty >= 0 && roty < ysize){
						vmem[x + ymul320] |= pic[2 + rotx + xsize * roty];
					}
				}
			}
			break;

		case 3:
			for(y = miny; y < maxy; y++){
				HANDLE_MATH_IN_Y_LOOP
				for(x = minx; x < maxx; x++){
					HANDLE_MATH_IN_X_LOOP
					if(rotx >= 0 && rotx < xsize && roty >= 0 && roty < ysize){
						vmem[x + ymul320] &= pic[2 + rotx + xsize * roty];
					}
				}
			}
			break;

		case 4:
			for(y = miny; y < maxy; y++){
				HANDLE_MATH_IN_Y_LOOP
				for(x = minx; x < maxx; x++){
					HANDLE_MATH_IN_X_LOOP
					if(rotx >= 0 && rotx < xsize && roty >= 0 && roty < ysize){
						if(pic[2 + rotx + xsize * roty])
							vmem[x + ymul320] = pic[2 + rotx + xsize * roty];
					}
				}
			}
			break;

		case 5:
			for(y = miny; y < maxy; y++){
				HANDLE_MATH_IN_Y_LOOP
				for(x = minx; x < maxx; x++){
					HANDLE_MATH_IN_X_LOOP
					if(rotx >= 0 && rotx < xsize && roty >= 0 && roty < ysize){
						if(!vmem[x + ymul320])
							vmem[x + ymul320] = pic[2 + rotx + xsize * roty];
					}
				}
			}
			break;

		case 6:
			for(y = miny; y < maxy; y++){
				HANDLE_MATH_IN_Y_LOOP
				for(x = minx; x < maxx; x++){
					HANDLE_MATH_IN_X_LOOP
					if(rotx >= 0 && rotx < xsize && roty >= 0 && roty < ysize){
						vmem[x + ymul320] += pic[2 + rotx + xsize * roty];
					}
				}
			}

	}
#undef HANDLE_MATH_IN_Y_LOOP
#undef HANDLE_MATH_IN_X_LOOP
}

void roterase(int x1, int y1, int x2, int y2, int centerx, int centery, float ang, float scale, unsigned char far *pic){
	const int SINE = sin(ang) * 64 / scale;
	const int COSINE = cos(ang) * 64 / scale;
	const int xsize = pic[0], ysize = pic[1];
	int deltax, deltay, rotx, roty, junk;
	int deltaymulSINE, deltaymulCOSINE, jumpsize;
	int leftdeltacenter, rightdeltacenter, bottomdeltacenter;
	unsigned char far *pptr;

	if(x1 < 0) x1 = 0; if(x1 > 319) x1 = 319;
	if(x2 < 0) x2 = 0; if(x2 > 319) x2 = 319;
	if(y1 < 0) y1 = 0; if(y1 > 199) y1 = 199;
	if(y2 < 0) y2 = 0; if(y2 > 199) y2 = 199;

	/****
		The ++'s are here so that I can use a < comparison in the for loops,
		rather than	a <=, which takes more processor time.
	****/
	x2++;
	y2++;

	/*
	pptr is the pointer aimed at the computer screen. I'm not too sure if
	it's any faster to increment pptr than it is to refer to vmem[n], but
	I think it is, and my testing didn't show any slowing down (although it
	didn't show any speeding up either).
	*/
	pptr = vmem + (y1 << 8) + (y1 << 6) + x1;

	/* jumpsize is the amount pptr is increased by to go from the end of
	   one line to the beginning of the of the next.  (rather than using
	   multiplication to calculate it)
	*/
	jumpsize = 320 - (x2 - x1);

	leftdeltacenter = x1 - centerx;
	rightdeltacenter = x2 - centerx;
	bottomdeltacenter = y2 - centery;
	for(deltay = y1 - centery; deltay < bottomdeltacenter; deltay++){

		deltaymulSINE = deltay * SINE;
		deltaymulCOSINE = deltay * COSINE;

		/***
			The reason why I have a leftdeltacenter in the x loop, and not a
			topdeltacenter in the y loop is this: The calculation of
			leftdeltacenter would be done y2 - y1 times if it was not assigned
			if a variable outside the loop.  On the other hand, topdeltacenter
			only gets calculated once, so assigning it to a variable would be
			a waste of time and memory.
		***/

		for(deltax = leftdeltacenter; deltax < rightdeltacenter; deltax++){
			rotx = deltax * COSINE - deltaymulSINE;
			roty = deltax * SINE + deltaymulCOSINE;

			rotx >>= 6;
			roty >>= 6;

			/****** add the rotated position to the center point ******/
			rotx += centerx;
			roty += centery;

			/****** modularize rotx & roty to fit the image ******/
			while(rotx < 0) rotx += xsize;
			while(roty < 0) roty += ysize;
			rotx %= xsize;
			roty %= ysize;

			*pptr = pic[2 + rotx + xsize * roty];
			pptr++;
		}

		pptr += jumpsize;
	}
}

void erase(int x1, int y1, int x2, int y2, int xoffset, int yoffset, unsigned char far *pic){
	int x, y, cx, cy, ymul320, xpicsize = pic[0], ypicsize = pic[1];

	if(x1 > x2) { x1 ^= x2; x2 ^= x1; x1 ^= x2; }
	if(y1 > y2) { y1 ^= y2; y2 ^= y1; y1 ^= y2; }

	if(y1 < 0) y1 = 0;
	if(y2 < 0) y2 = 0;
	if(y1 > 199) y1 = 199;
	if(y2 > 199) y2 = 199;
	if(x1 < 0) x1 = 0;
	if(x2 < 0) x2 = 0;
	if(x1 > 319) x1 = 319;
	if(x2 > 319) x2 = 319;


	for(y = y1; y <= y2; y++){
		cy = (y + yoffset) % ypicsize;
		ymul320 = (y << 8) + (y << 6);

		for(x = x1; x <= x2; x++){
			cx = (x + xoffset) % xpicsize;
			vmem[ymul320 + x] = pic[cx + xpicsize * cy + 2];
		}
	}
}

char touching(int x1, int y1, char *pic1, int x2, int y2, char *pic2, char mode){
	int x, y;
	char returnval = 0;

	if((x1 >= x2 - pic1[0]) &&
	  (x1 <= x2 + pic2[0]) &&
	  (y1 >= y2 - pic1[1]) &&
	  (y1 <= y2 + pic2[1]))
	{
		if(!mode) returnval = 1;
		else{
			for(x = x1; x < x1 + pic1[0]; x++){
				for(y = y1; y < y1 + pic1[1]; y++){
					if((x >= x2) && (x <= x2 + pic2[0] - 1) &&
					  (y >= y2) && (y <= y2 + pic2[1] - 1))
					{
						if(pic1[2 + x - x1 + pic1[0] * (y - y1)] &&
						   pic2[2 + x - x2 + pic2[0] * (y - y2)])
							returnval = 1;
					}
				}
			}
		}

	}

	return returnval;
}

/****************************** File I/O ******************************/
void writebit(FILE *fout, char bitval){
	static unsigned char bit[8], numbits = 0;
	unsigned char byte, n;

	if(bitval == -1){ /* force write despite numbits < 8 */
		for(;numbits < 8; numbits++)
			bit[numbits] = 0;
	}else{
		bit[numbits] = (bitval != 0);
		numbits++;
	}

	if(numbits >= 8){
		byte = 0;
		for(n = 0; n < 8; n++){
			if(bit[n])
				byte += 1 << (7 - n);
		}
		fputc(byte, fout);
		numbits = 0;
	}
}

char readbit(FILE *fin){
	char returnval = 0;
	static unsigned char byte, bitsleft = 0;


	if(!feof(fin) || bitsleft){
		if(bitsleft == 0){
			byte = fgetc(fin);
			bitsleft = 8;
		}

		bitsleft--;
		if(byte >=  1 << bitsleft){
			byte -= (1 << bitsleft);
			returnval = 1;
		}
		return returnval;
	}else{
		return -1;
	}
}

void save_font(unsigned char *name){
	int c, n;
	FILE *fout;

	if(font){
		fout = fopen(name, "wb");

		fputc(xfontsize, fout);
		fputc(yfontsize, fout);

		for(c = 0; c < 256; c++){
			for(n = 2; n < (xfontsize * yfontsize + 2); n++){
				//fputc(font[c][n], fout);
				writebit(fout, font[c][n] != 0);
			}
		}
		writebit(fout, -1);

		fclose(fout);
	}
}

void unload_font(void){
	int n;
	if(font){
		for(n = 0; n < 256; n++)
			if(font[n])
				free(font[n]);
		free(font);
		font = NULL;
	}
}

char load_font(unsigned char *name){
	int c, n, bad = 0;
	FILE *fname;
	fname = fopen(name, "rb");

	xfontsize = fgetc(fname);
	yfontsize = fgetc(fname);

	/******* free previously loaded font *******/
	if(font != NULL) unload_font();

	/******** allocate memory for new font *******/
	font = (unsigned char **)malloc(256 * sizeof(char *));
	if(font){
		for(n = 0; n < 256; n++){
			font[n] = (char *)malloc(xfontsize * yfontsize + 2);
			if(!font[n]) bad = 1;
		}
		if(bad){
			for(n = 0; n < 256; n++)
				if(font[n]) free(font[n]);
			free(font);
			font = NULL;
		}
	}else bad = 1;

	/******** load new font **********/
	for(c = 0; (c < 256) && !bad; c++){
		font[c][0] = xfontsize;
		font[c][1] = yfontsize;
		for(n = 2; n < (xfontsize * yfontsize + 2); n++){
			font[c][n] = readbit(fname);
		}
	}
	fclose(fname);

	return (!bad);
}

void savepic(char far *image, int arraysize, unsigned char *filename){
	int n;
	FILE *picfile;
	picfile = fopen(filename, "wb");
	n = image[0] * image[1] + 2;
	if(arraysize < n) n = arraysize;
	fwrite((void *)image, 1, n, picfile);
	fclose(picfile);
}

void loadpic(char far *image, int arraysize, unsigned char *filename){
	int n, size;
	FILE *picfile;
	picfile = fopen(filename, "rb");

	image[0] = fgetc(picfile);
	image[1] = fgetc(picfile);
	size = image[0] * image[1] + 2;
	if(size > arraysize) size = arraysize;
	size -= 2;
	fread((void *)&image[2], 1, size, picfile);
	fclose(picfile);
}

/**************************** user I/O *******************************/
unsigned char choose(unsigned char *title, char numbuttons, ...){
	typedef struct{
		char *text, character;
		int x1, x2, y1, y2;
	}buttontype;

	unsigned char memgood, *underbox, fini = 0;
	unsigned char keypressed, buttonused;
	buttontype *button;
	va_list arg_list;
	int buttonswidth, titlewidth;
	int n, boxwidth, boxx1, boxx2, boxy1, boxy2, x, y;

	/***** allocate enough memory to store all of the buttons *****/
	n = (4 * sizeof(int) + sizeof(char *) + sizeof(char)) * numbuttons;
	button = (buttontype *)malloc(n);
	memgood = ((button != NULL) || (n == 0));

	/***** read the various arguments *****/
	va_start(arg_list, numbuttons);
	for(n = 0; n < numbuttons; n++){
		button[n].text = strdup(va_arg(arg_list, char *));
		button[n].character = va_arg(arg_list, char);
		if(button[n].text == NULL) memgood = 0;
	}
	va_end(arg_list);

	/***** calculate collective width of buttons *****/
	if(memgood){
		buttonswidth = xfontsize;
		for(n = 0; n < numbuttons; n++){
			buttonswidth += (strlen(button[n].text) + 1) * xfontsize;
		}
	}
	boxwidth = buttonswidth;

	titlewidth = (strlen(title) + 2) * xfontsize;
	if(boxwidth < titlewidth) boxwidth = titlewidth;

	/***** calculate size and location of box *****/
	boxx1 = 160 - (boxwidth / 2);
	boxx2 = boxx1 + boxwidth - 1;
	boxy1 = 100 - 2 * yfontsize;
	boxy2 = boxy1 + 4 * yfontsize - 1;

	underbox = (char *)malloc((boxx2 - boxx1 + 1) * (boxy2 - boxy1 + 1) + 2);
	memgood *= (underbox != NULL);

	if(memgood){
		/***** get the image beneath the box *****/
		showmouse(0);
		get(boxx1, boxy1, boxx2, boxy2, underbox);

		/***** draw the box *****/
		box(boxx1, boxy1, boxx2 - 1, boxy2 - 1, 22, 4);
		box(boxx1 + 1, boxy1 + 1, boxx2, boxy2, 18, 4);
		box(boxx1 + 1, boxy1 + 1, boxx2 - 1, boxy2 - 1, 20, 0);

		/***** determine the locations of, and draw, the buttons *****/
		x = 160 - buttonswidth / 2;
		for(n = 0; n < numbuttons; n++){
			x += xfontsize;
			button[n].x1 = x - 2;
			x += (xfontsize * strlen(button[n].text));
			button[n].x2 = x + 2;
			button[n].y2 = boxy2 - (yfontsize / 2) + 1;
			button[n].y1 = button[n].y2 - yfontsize - 2;
			box(button[n].x1 + 1, button[n].y1 + 1, button[n].x2, button[n].y2, 18, 4);
			box(button[n].x1, button[n].y1, button[n].x2 - 1, button[n].y2 - 1, 22, 4);
			box(button[n].x1 + 1, button[n].y1 + 1, button[n].x2 - 1, button[n].y2 - 1, 20, 0);
			drawtext(button[n].x1 + 2, button[n].y1 + 1, button[n].text, 15, 4);
		}

		/***** draw the title *****/
		drawtext(160 - titlewidth / 2 + xfontsize, boxy1 + yfontsize / 2, title, 15, 4);

		showmouse(1);
		while(!fini){
			/***** handle keyboard IO *****/
			if(kbhit()){

				keypressed = getch();

				/***** test for hotkeys *****/
				for(n = 0; n < numbuttons && !fini; n++){
					if(keypressed == button[n].character){
						fini = 1;
						buttonused = n;
					}
				}
			}

			/***** handle mouse IO *****/
			if(buttonpressed()){
				x = mousex();
				y = mousey();
				for(n = 0; n < numbuttons; n++){
					if((button[n].x1 <= x) && (button[n].x2 >= x) &&
					   (button[n].y1 <= y) && (button[n].y2 >= y))
						break;
				}
				if(n < numbuttons){
					fini = 1;
					buttonused = n;
				}
			}
		}
		showmouse(0);
		put(boxx1, boxy1, underbox, 0);
		showmouse(1);
	}

	for(n = 0; n < numbuttons; n++)
		if(button[n].text) free(button[n].text);
	if(button) free(button);
	if(underbox) free(underbox);

	return buttonused;
}

unsigned char *taketext(unsigned char *title, char *buttonused, char numbuttons, ...){
	typedef struct{
		char *text, character;
		int x1, x2, y1, y2;
	}buttontype;

	const int maxstringlength = 1024;
	unsigned char returnval[1024], memgood, *underbox, fini = 0;
	unsigned char keypressed, *drawstring, insertmode = 1;
	buttontype *button;
	va_list arg_list;
	int buttonswidth, titlewidth;
	int n, boxwidth, boxx1, boxx2, boxy1, boxy2, x, y;
	int textx1, textx2, texty, displaywidth, stringlength;
	int coffset, doffset, cx;

	/***** allocate enough memory to store all of the buttons *****/
	n = (4 * sizeof(int) + sizeof(char *) + sizeof(char)) * numbuttons;
	button = (buttontype *)malloc(n);
	memgood = ((button != NULL) || (n == 0));

	/***** read the various arguments *****/
	va_start(arg_list, numbuttons);
	for(n = 0; n < numbuttons; n++){
		button[n].text = strdup(va_arg(arg_list, char *));
		button[n].character = va_arg(arg_list, char);
		if(button[n].text == NULL) memgood = 0;
	}
	va_end(arg_list);

	/***** calculate collective width of buttons *****/
	if(memgood){
		buttonswidth = xfontsize;
		for(n = 0; n < numbuttons; n++){
			buttonswidth += (strlen(button[n].text) + 1) * xfontsize;
		}
	}
	boxwidth = buttonswidth;

	titlewidth = (strlen(title) + 2) * xfontsize;
	if(boxwidth < titlewidth) boxwidth = titlewidth;

	/***** calculate size and location of box *****/
	boxx1 = 160 - (boxwidth / 2);
	boxx2 = boxx1 + boxwidth - 1;
	if(numbuttons){
		boxy1 = 100 - 3 * yfontsize;
		boxy2 = boxy1 + 6 * yfontsize - 1;
	}else{
		boxy1 = 100 - 2 * yfontsize;
		boxy2 = boxy1 + 4 * yfontsize - 1;
	}

	underbox = (char *)malloc((boxx2 - boxx1 + 1) * (boxy2 - boxy1 + 1) + 2);
	memgood *= (underbox != NULL);

	if(memgood){
		/***** get the image beneath the box *****/
		get(boxx1, boxy1, boxx2, boxy2, underbox);

		/***** draw the box *****/
		box(boxx1, boxy1, boxx2 - 1, boxy2 - 1, 22, 4);
		box(boxx1 + 1, boxy1 + 1, boxx2, boxy2, 18, 4);
		box(boxx1 + 1, boxy1 + 1, boxx2 - 1, boxy2 - 1, 20, 0);

		/***** determine the locations of, and draw, the buttons *****/
		x = 160 - buttonswidth / 2;
		for(n = 0; n < numbuttons; n++){
			x += xfontsize;
			button[n].x1 = x - 2;
			x += (xfontsize * strlen(button[n].text));
			button[n].x2 = x + 2;
			button[n].y2 = boxy2 - (yfontsize / 2) + 1;
			button[n].y1 = button[n].y2 - yfontsize - 2;
			box(button[n].x1 + 1, button[n].y1 + 1, button[n].x2, button[n].y2, 18, 4);
			box(button[n].x1, button[n].y1, button[n].x2 - 1, button[n].y2 - 1, 22, 4);
			box(button[n].x1 + 1, button[n].y1 + 1, button[n].x2 - 1, button[n].y2 - 1, 20, 0);
			drawtext(button[n].x1 + 2, button[n].y1 + 1, button[n].text, 15, 4);
		}

		/***** draw the title *****/
		drawtext(160 - titlewidth / 2 + xfontsize, boxy1 + yfontsize / 2, title, 15, 4);

		/***** draw the text box *****/
		n = boxx2 - boxx1 - 2 * xfontsize;
		n -= (n % xfontsize);
		textx1 = 160 - n / 2;
		textx2 = textx1 + n;
		displaywidth = n / xfontsize;
		drawstring = (char *)malloc(displaywidth + 1);
		memgood = drawstring != NULL;

		if(numbuttons)
			texty = button[0].y1 - 1.5 * yfontsize;
		else
			texty = boxy2 - 1.5 * yfontsize;
		box(textx1 - 2, texty - 2, textx2 + 1, texty + yfontsize + 1, 18, 0);
		box(textx1 - 1, texty - 1, textx2 + 2, texty + yfontsize + 2, 22, 0);
		box(textx1 - 1, texty - 1, textx2 + 1, texty + yfontsize + 1, 0, 0);

		showmouse(1);
		memset(returnval, NULL, maxstringlength);
		coffset = doffset = stringlength = 0;
		while(!fini && drawstring){
			/***** handle keyboard IO *****/
			if(kbhit()){
				/***** hide the cursor *****/
				box(textx1 + xfontsize * (stringlength + coffset - doffset), texty,
					textx1 + xfontsize * (stringlength + coffset - doffset) + (xfontsize - 2) * (1 - insertmode), texty + yfontsize - 1,
					0, 0);

				keypressed = getch();

				/***** test for hotkeys *****/
				for(n = 0; n < numbuttons && !fini; n++){
					if(keypressed == button[n].character){
						fini = 1;
						*buttonused = n;
					}
				}
				/***** test for other keys *****/
				if(!fini){
					switch(keypressed){
						case 0:
							switch(getch()){
								case 'G': //home
									coffset = -stringlength;
								break;
								case 'O':
									coffset = 0;
								break;
								case 'K': //left
									if(- coffset < stringlength)
										coffset--;
								break;
								case 'M': //right
									if(coffset < 0)
										coffset++;
								break;
								case 83: //del
									if(- coffset > 0){
										for(n = coffset; n < 0; n++)
											returnval[stringlength + n] = returnval[stringlength + n + 1];
										stringlength--;
										coffset++;
										returnval[stringlength] = NULL;
									}
								break;
								case 82: //insert
									insertmode = 1 - insertmode;
								break;
							}
						break;
						case 8: //backspace
							if(- coffset < stringlength){
								for(n = coffset; n < 0; n++)
									returnval[stringlength + n - 1] = returnval[stringlength + n];
								stringlength--;
								returnval[stringlength] = NULL;
							}
						break;
						case 13: //enter
							*buttonused = 0;
							fini = 1;
						break;
						default: //alphanumagrammatic characters
							if((keypressed >= 32) && (keypressed <= 126) && (stringlength < maxstringlength)){
								if(insertmode)
									for(n = 0; n > coffset; n--)
										returnval[stringlength + n] = returnval[stringlength + n - 1];

								returnval[stringlength + coffset] = keypressed;

								if(insertmode || coffset == 0)
									stringlength++;
								else
									coffset++;
							}
					}
				}

				/***** redraw the string *****/
				n = stringlength + coffset;
				if(n > doffset + displaywidth - !insertmode)
					doffset = n - displaywidth + !insertmode;
				if(n < doffset)
					doffset = n;

				memset(drawstring, 32, displaywidth + 1);
				drawstring[displaywidth] = NULL;
				strncpy(drawstring, &returnval[doffset], displaywidth);
				for(n = 0; n < displaywidth; n++)
					if(drawstring[n] == NULL) drawstring[n] = 32;

				showmouse(0);
				drawtext(textx1, texty, drawstring, 15, 0);

				/***** draw the cursor *****/
				box(textx1 + xfontsize * (stringlength + coffset - doffset), texty,
					textx1 + xfontsize * (stringlength + coffset - doffset) + (xfontsize - 2) * (1 - insertmode), texty + yfontsize - 1,
					14, 1);
				showmouse(1);
			}

			/***** handle mouse IO *****/
			if(buttonpressed()){
				x = mousex();
				y = mousey();
				for(n = 0; n < numbuttons; n++){
					if((button[n].x1 <= x) && (button[n].x2 >= x) &&
					   (button[n].y1 <= y) && (button[n].y2 >= y))
						break;
				}
				if(n < numbuttons){
					fini = 1;
					*buttonused = n;
				}
			}
		}
		showmouse(0);

		put(boxx1, boxy1, underbox, 0);
	}

	for(n = 0; n < numbuttons; n++)
		if(button[n].text) free(button[n].text);
	if(button) free(button);
	if(underbox) free(underbox);
	if(drawstring) free(drawstring);

	return returnval;
}

char choice(char *title, int numoptions, ...){
	int n, x1[50], x2[50], y1[50], y2[50], tx, ty, mx, my;
	int bx1, by1, bx2, by2;
	unsigned char underpointer[122], *option[50];
	char fini = 0, copt = 0, returnval, option_chosen = -1;
	unsigned char pointer[122];
	va_list ap;

	va_start(ap, numoptions);
	for(n = 0; n < numoptions; n++){
		option[n] = va_arg(ap, char *);
	}
	va_end(ap);
	loadpic(pointer, 122, "POINTER.PIC");


	/********* draw the title **********/
	tx = 160 - (xfontsize * strlen(title)) / 2;
	ty = 100 - 2 * yfontsize;

	x1[0] = 160;
	for(n = 0; n < numoptions; n++)
		x1[0] -= (option[n][0] + 10) / 2;
	x2[0] = x1[0] + option[0][0] - 1;
	y1[0] = 100 - option[0][1] / 2 + 2 * yfontsize;
	y2[0] = y1[0] + option[0][1] - 1;

	for(n = 1; n < numoptions; n++){
		x1[n] = x2[n - 1] + 10;
		x2[n] = x1[n] + option[n][0] - 1;
		y1[n] = 100 - option[n][1] / 2 + 2 * yfontsize;
		y2[n] = y1[n] + option[n][1] - 1;
	}

	bx1 = tx - 10;
	bx2 = tx + strlen(title) * xfontsize + 10;
	by1 = ty - 10;
	by2 = ty + yfontsize;
	for(n = 0; n < numoptions; n++){
		if(x1[n] - 10 < bx1) bx1 = x1[n] - 10;
		if(x2[n] + 10 > bx2) bx2 = x2[n] + 10;
		if(y1[n] - 10 < by1) by1 = y1[n] - 10;
		if(y2[n] + 10 > by2) by2 = y2[n] + 10;
	}

	box(bx1 - 1, by1 - 1, bx2, by2, 22, 4);
	box(bx1, by1, bx2 + 1, by2 + 1, 18, 4);
	box(bx1, by1, bx2, by2, 20, 0);

	drawtext(tx, ty, title, 15, 4);
	for(n = 0; n < numoptions; n++)
		put(x1[n], y1[n], option[n], 0);


	while(kbhit()) getch();
	box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 14, 4);
	mx = mousex();
	my = mousey();
	get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
	while(!fini){
		/******** handle mouse IO **********/
		if(mx != mousex() || my != mousey()){
			put(mx, my, underpointer, 0);
			mx = mousex();
			my = mousey();
			for(n = 0; n < numoptions; n++){
				if(mx >= x1[n] && mx <= x2[n] && my >= y1[n] && my <= y2[n] && n != copt){
					box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 20, 4);
					copt = n;
					box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 14, 4);
				}
			}
			get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
			put(mx, my, pointer, 4);
		}
		option_chosen = -1;
		while(buttonpressed()){
			if(mx != mousex() || my != mousey()){
				put(mx, my, underpointer, 0);
				mx = mousex();
				my = mousey();
				get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
				put(mx, my, pointer, 4);
			}
			option_chosen = -1;
			for(n = 0; n < numoptions; n++){
				if(mx >= x1[n] && mx <= x2[n] && my >= y1[n] && my <= y2[n]){
					if(n != copt){
						put(mx, my, underpointer, 0);
						box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 20, 4);
						copt = n;
						box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 14, 4);
						get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
						put(mx, my, pointer, 4);
					}
					option_chosen = n;
				}
			}
		}
		if(option_chosen != -1){
			returnval = option_chosen;
			fini = 1;
		}

		/******** handle keyboard IO ********/
		if(kbhit()){
			switch(getch()){
				case 13:
					returnval = copt;
					fini = 1;
				break;
				case 0:
					switch(getch()){
						case 'H': case 'K':
							put(mx, my, underpointer, 0);
							box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 20, 4);
							copt--;
							if(copt < 0) copt += numoptions;
							box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 14, 4);
							get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
						break;
						case 'M': case 'P':
							put(mx, my, underpointer, 0);
							box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 20, 4);
							copt = (copt + 1) % numoptions;
							box(x1[copt] - 1, y1[copt] - 1, x2[copt] + 1, y2[copt] + 1, 14, 4);
							get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
						break;
					}
				break;
			}
		}
	}
	put(mx, my, underpointer, 0);
	return returnval;
}

char menuscreen(char *title, char *background, int numoptions, ...){
	int n, x1[50], x2[50], y1[50], y2[50], tx, ty, mx, my;
	char *option[50], fini = 0, copt = 0, returnval;
	char option_chosen = -1;
	unsigned char underpointer[122], pointer[122], pic[2000];
	va_list ap;

	loadpic(pointer, 122, "POINTER.PIC");
	va_start(ap, numoptions);
	for(n = 0; n < numoptions; n++){
		option[n] = va_arg(ap, char *);
	}
	va_end(ap);

	loadpic(pic, 2000, background);
	erase(0, 0, 319, 199, 0, 0, pic);

	/********* draw the title **********/
	tx = 160 - (xfontsize * strlen(title)) / 2;
	ty = 15;
	box(tx - 24, ty - 9, tx + 26+ strlen(title) * xfontsize - 1, ty + 11 + yfontsize - 1, 16, 0);
	box(tx - 26, ty - 11, tx + 24 + strlen(title) * xfontsize - 1, ty + 9 + yfontsize - 1, 20, 0);
	box(tx - 25, ty - 10, tx + 25 + strlen(title) * xfontsize - 1, ty + 10 + yfontsize - 1, 18, 0);
	drawtext(tx, ty, title, 15, 4);

	for(n = 0; n < numoptions; n++){
		x1[n] = 160 - (strlen(option[n]) * xfontsize) / 2 - xfontsize / 2;
		x2[n] = x1[n] + strlen(option[n]) * xfontsize + xfontsize;
		y1[n] = 130 + (n - (numoptions / 2)) * 2 * yfontsize - yfontsize / 3;
		y2[n] = y1[n] + yfontsize + 2 * yfontsize / 3;
		box(x1[n] + 1, y1[n] + 1, x2[n], y2[n], 18, 0);
		box(x1[n], y1[n], x2[n] - 1, y2[n] - 1, 26, 0);
		box(x1[n] + 1, y1[n] + 1, x2[n] - 1, y2[n] - 1, 22, 0);
		drawtext(x1[n] + xfontsize / 2, y1[n] + yfontsize / 3, option[n], 15, 4);

	}

	while(kbhit()) getch();
	box(x1[copt], y1[copt], x2[copt], y2[copt], 4, 6);
	mx = mousex();
	my = mousey();
	get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
	while(!fini){
		/******** handle mouse IO **********/
		if(mx != mousex() || my != mousey()){
			put(mx, my, underpointer, 0);
			mx = mousex();
			my = mousey();
			for(n = 0; n < numoptions; n++){
				if(mx >= x1[n] && mx <= x2[n] && my >= y1[n] && my <= y2[n] && n != copt){
					box(x1[copt], y1[copt], x2[copt], y2[copt], -4, 6);
					copt = n;
					box(x1[copt], y1[copt], x2[copt], y2[copt], 4, 6);
				}
			}
			get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
			put(mx, my, pointer, 4);
		}
		option_chosen = -1;
		while(buttonpressed()){
			if(mx != mousex() || my != mousey()){
				put(mx, my, underpointer, 0);
				mx = mousex();
				my = mousey();
				get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
				put(mx, my, pointer, 4);
			}
			option_chosen = -1;
			for(n = 0; n < numoptions; n++){
				if(mx >= x1[n] && mx <= x2[n] && my >= y1[n] && my <= y2[n]){
					if(n != copt){
						put(mx, my, underpointer, 0);
						box(x1[copt], y1[copt], x2[copt], y2[copt], -4, 6);
						copt = n;
						box(x1[copt], y1[copt], x2[copt], y2[copt], 4, 6);
						get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
						put(mx, my, pointer, 4);
					}
					option_chosen = n;
				}
			}
		}
		if(option_chosen != -1){
			returnval = option_chosen;
			fini = 1;
		}

		/******** handle keyboard IO ********/
		if(kbhit()){
			switch(getch()){
				case 13:
					returnval = copt;
					fini = 1;
				break;
				case 0:
					switch(getch()){
						case 'H': case 'K':
							put(mx, my, underpointer, 0);
							box(x1[copt], y1[copt], x2[copt], y2[copt], -4, 6);
							copt--;
							if(copt < 0) copt += numoptions;
							box(x1[copt], y1[copt], x2[copt], y2[copt], 4, 6);
							get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
						break;
						case 'M': case 'P':
							put(mx, my, underpointer, 0);
							box(x1[copt], y1[copt], x2[copt], y2[copt], -4, 6);
							copt = (copt + 1) % numoptions;
							box(x1[copt], y1[copt], x2[copt], y2[copt], 4, 6);
							get(mx, my, mx + pointer[0] - 1, my + pointer[1] - 1, underpointer);
						break;
					}
				break;
			}
		}
	}
	put(mx, my, underpointer, 0);
	return returnval;
}